@@ -1,27 +1,25 @@
 /*
     SDL_image:  An example image loading library for use with SDL
-    Copyright (C) 1999-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
 
-/* $Id: IMG_pcx.c,v 1.10 2004/01/04 22:04:38 slouken Exp $ */
-
 /*
  * PCX file reader:
  * Supports:
@@ -64,12 +62,14 @@ struct PCXheader {
 /* See if an image is contained in a data source */
 int IMG_isPCX(SDL_RWops *src)
 {
+	int start;
 	int is_PCX;
 	const int ZSoft_Manufacturer = 10;
 	const int PC_Paintbrush_Version = 5;
 	const int PCX_RunLength_Encoding = 1;
 	struct PCXheader pcxh;
 
+	start = SDL_RWtell(src);
 	is_PCX = 0;
 	if ( SDL_RWread(src, &pcxh, sizeof(pcxh), 1) == 1 ) {
 		if ( (pcxh.Manufacturer == ZSoft_Manufacturer) &&
@@ -78,12 +78,14 @@ int IMG_isPCX(SDL_RWops *src)
 			is_PCX = 1;
 		}
 	}
+	SDL_RWseek(src, start, SEEK_SET);
 	return(is_PCX);
 }
 
 /* Load a PCX type image from an SDL datasource */
 SDL_Surface *IMG_LoadPCX_RW(SDL_RWops *src)
 {
+	int start;
 	struct PCXheader pcxh;
 	Uint32 Rmask;
 	Uint32 Gmask;
@@ -100,6 +102,8 @@ SDL_Surface *IMG_LoadPCX_RW(SDL_RWops *src)
 		/* The error message has been set in SDL_RWFromFile */
 		return NULL;
 	}
+	start = SDL_RWtell(src);
+
 	if ( ! SDL_RWread(src, &pcxh, sizeof(pcxh), 1) ) {
 		error = "file truncated";
 		goto done;
@@ -232,9 +236,12 @@ SDL_Surface *IMG_LoadPCX_RW(SDL_RWops *src)
 done:
 	free(buf);
 	if ( error ) {
-		SDL_FreeSurface(surface);
+		SDL_RWseek(src, start, SEEK_SET);
+		if ( surface ) {
+			SDL_FreeSurface(surface);
+			surface = NULL;
+		}
 		IMG_SetError(error);
-		surface = NULL;
 	}
 	return(surface);
 }
